from datetime import datetime
from enum import Enum

from checkout_sdk.common.enums import Currency
from checkout_sdk.payments.payments import PaymentType, BillingDescriptor, PaymentCustomerRequest, ShippingDetails, \
    PaymentRecipient, ProcessingSettings, RiskRequest, ThreeDsRequest, PaymentSender


class PaymentMethodsType(str, Enum):
    ALIPAY_CN = 'alipay_cn'
    ALIPAY_HK = 'alipay_hk'
    ALMA = 'alma'
    APPLEPAY = 'applepay'
    BANCONTACT = 'bancontact'
    BENEFIT = 'benefit'
    BIZUM = 'bizum'
    CARD = 'card'
    DANA = 'dana'
    EPS = 'eps'
    GCASH = 'gcash'
    GOOGLEPAY = 'googlepay'
    IDEAL = 'ideal'
    KAKAOPAY = 'kakaopay'
    KLARNA = 'klarna'
    KNET = 'knet'
    MBWAY = 'mbway'
    MOBILEPAY = 'mobilepay'
    MULTIBANCO = 'multibanco'
    P24 = 'p24'
    PAYPAL = 'paypal'
    PLAID = 'plaid'
    QPAY = 'qpay'
    REMEMBER_ME = 'remember_me'
    SEPA = 'sepa'
    STCPAY = 'stcpay'
    STORED_CARD = 'stored_card'
    TABBY = 'tabby'
    TAMARA = 'tamara'
    TNG = 'tng'
    TRUEMONEY = 'truemoney'
    TWINT = 'twint'
    VIPPS = 'vipps'


class Locale(str, Enum):
    AR = 'ar'
    DA_DK = 'da-DK'
    DE_DE = 'de-DE'
    EL = 'el'
    EN_GB = 'en-GB'
    ES_ES = 'es-ES'
    FI_FI = 'fi-FI'
    FIL_PH = 'fil-PH'
    FR_FR = 'fr-FR'
    HI_IN = 'hi-IN'
    ID_ID = 'id-ID'
    IT_IT = 'it-IT'
    JA_JP = 'ja-JP'
    KO_KR = 'ko-KR'
    MS_MY = 'ms-MY'
    NB_NO = 'nb-NO'
    NL_NL = 'nl-NL'
    PT_PT = 'pt-PT'
    SV_SE = 'sv-SE'
    TH_TH = 'th-TH'
    VI_VN = 'vi-VN'
    ZH_CN = 'zh-CN'
    ZH_HK = 'zh-HK'
    ZH_TW = 'zh-TW'


class StorePaymentDetails(str, Enum):
    DISABLED = 'disabled'
    ENABLED = 'enabled'
    COLLECT_CONSENT = 'collect_consent'


class InstructionPurpose(str, Enum):
    DONATIONS = 'donations'
    EDUCATION = 'education'
    EMERGENCY_NEED = 'emergency_need'
    EXPATRIATION = 'expatriation'
    FAMILY_SUPPORT = 'family_support'
    FINANCIAL_SERVICES = 'financial_services'
    GIFTS = 'gifts'
    INCOME = 'income'
    INSURANCE = 'insurance'
    INVESTMENT = 'investment'
    IT_SERVICES = 'it_services'
    LEISURE = 'leisure'
    LOAN_PAYMENT = 'loan_payment'
    MEDICAL_TREATMENT = 'medical_treatment'
    OTHER = 'other'
    PENSION = 'pension'
    ROYALTIES = 'royalties'
    SAVINGS = 'savings'
    TRAVEL_AND_TOURISM = 'travel_and_tourism'


class Instruction:
    purpose: InstructionPurpose


class CustomerRetry:
    max_attempts: int


class AccountHolderType(str, Enum):
    INDIVIDUAL = 'individual'
    CORPORATE = 'corporate'
    GOVERNMENT = 'government'


class BillingAddress:
    country: str
    address_line1: str
    address_line2: str
    city: str
    state: str
    zip: str


class BillingPhone:
    country_code: str
    number: str


class SessionBilling:
    address: BillingAddress
    phone: BillingPhone


class AccountHolder:
    type: AccountHolderType

    def __init__(self, type_p: AccountHolderType):
        self.type = type_p


class IndividualAccountHolder(AccountHolder):
    first_name: str
    last_name: str
    middle_name: str
    account_name_inquiry: bool

    def __init__(self):
        super().__init__(AccountHolderType.INDIVIDUAL)


class CorporateAccountHolder(AccountHolder):
    company_name: str
    account_name_inquiry: bool

    def __init__(self):
        super().__init__(AccountHolderType.CORPORATE)


class ApplePayConfiguration:
    store_payment_details: StorePaymentDetails
    account_holder: AccountHolder


class CardConfiguration:
    store_payment_details: StorePaymentDetails
    account_holder: AccountHolder


class GooglePayConfiguration:
    store_payment_details: StorePaymentDetails
    account_holder: AccountHolder


class StoredCardConfiguration:
    customer_id: str
    instrument_ids: list  # list of strings
    default_instrument_id: str


class SessionPaymentMethodConfiguration:
    applepay: ApplePayConfiguration
    card: CardConfiguration
    googlepay: GooglePayConfiguration
    stored_card: StoredCardConfiguration


class Item:
    name: str
    quantity: int
    unit_price: int
    reference: str
    commodity_code: str
    unit_of_measure: str
    total_amount: int
    tax_amount: int
    discount_amount: int
    url: str
    image_url: str


class PaymentSessionsRequest:
    amount: int
    currency: Currency
    billing: SessionBilling
    success_url: str
    failure_url: str
    payment_type: PaymentType
    billing_descriptor: BillingDescriptor
    reference: str
    description: str
    customer: PaymentCustomerRequest
    shipping: ShippingDetails
    recipient: PaymentRecipient
    processing: ProcessingSettings
    instruction: Instruction
    processing_channel_id: str
    payment_method_configuration: SessionPaymentMethodConfiguration
    items: list  # Item
    amount_allocations: list  # AmountAllocations
    risk: RiskRequest
    display_name: str
    metadata: dict
    locale: str
    three_ds: ThreeDsRequest
    sender: PaymentSender
    capture: bool
    capture_on: datetime
    expires_on: datetime
    enabled_payment_methods: list  # PaymentMethodsType
    disabled_payment_methods: list  # PaymentMethodsType
    customer_retry: CustomerRetry
    ip_address: str


class PaymentSessionWithPaymentRequest:
    session_data: str
    amount: int
    currency: Currency
    billing: SessionBilling
    success_url: str
    failure_url: str
    payment_type: PaymentType
    billing_descriptor: BillingDescriptor
    reference: str
    description: str
    customer: PaymentCustomerRequest
    shipping: ShippingDetails
    recipient: PaymentRecipient
    processing: ProcessingSettings
    instruction: Instruction
    processing_channel_id: str
    payment_method_configuration: SessionPaymentMethodConfiguration
    items: list  # Item
    amount_allocations: list  # AmountAllocations
    risk: RiskRequest
    display_name: str
    metadata: dict
    locale: str
    three_ds: ThreeDsRequest
    sender: PaymentSender
    capture: bool
    capture_on: datetime


class SubmitPaymentSessionRequest:
    session_data: str
    amount: int
    reference: str
    items: list  # Item
    three_ds: ThreeDsRequest
    ip_address: str



from __future__ import absolute_import

from checkout_sdk.authorization_type import AuthorizationType
from checkout_sdk.exception import CheckoutAuthorizationException
from checkout_sdk.platform_type import PlatformType
from checkout_sdk.sdk_authorization import SdkAuthorization
from checkout_sdk.sdk_credentials import SdkCredentials


class PreviousKeysSdkCredentials(SdkCredentials):

    def __init__(self, secret_key, public_key=None):
        self.secret_key = secret_key
        self.public_key = public_key

    def get_authorization(self, authorization_type: AuthorizationType):
        if AuthorizationType.SECRET_KEY == authorization_type:
            if self.secret_key is None:
                raise CheckoutAuthorizationException.invalid_key(AuthorizationType.SECRET_KEY)
            return SdkAuthorization(PlatformType.PREVIOUS, self.secret_key)
        if AuthorizationType.PUBLIC_KEY == authorization_type:
            if self.public_key is None:
                raise CheckoutAuthorizationException.invalid_key(AuthorizationType.PUBLIC_KEY)
            return SdkAuthorization(PlatformType.PREVIOUS, self.public_key)
        raise CheckoutAuthorizationException.invalid_authorization(authorization_type=authorization_type)



from enum import Enum


class ControlType(str, Enum):
    VELOCITY_LIMIT = 'velocity_limit'
    MCC_LIMIT = 'mcc_limit'


class VelocityWindowType(str, Enum):
    DAILY = 'daily'
    WEEKLY = 'weekly'
    MONTHLY = 'monthly'
    ALL_TIME = 'all_time'


class MccLimitType(str, Enum):
    ALLOW = 'allow'
    BLOCK = 'block'


class VelocityWindow:
    type: VelocityWindowType


class VelocityLimit:
    amount_limit: int
    velocity_window: VelocityWindow
    mcc_list: list  # str


class MccLimit:
    type: MccLimitType
    mcc_list: list  # str


class CardControlRequest:
    description: str
    control_type: ControlType
    target_id: str

    def __init__(self, control_type: ControlType):
        self.control_type = control_type


class VelocityControlRequest(CardControlRequest):
    velocity_limit: VelocityLimit

    def __init__(self):
        super().__init__(ControlType.VELOCITY_LIMIT)


class MccControlRequest(CardControlRequest):
    mcc_limit: MccLimit

    def __init__(self):
        super().__init__(ControlType.MCC_LIMIT)


class CardControlsQuery:
    target_id: str


class UpdateCardControlRequest:
    description: str
    velocity_limit: VelocityLimit
    mcc_limit: MccLimit



from __future__ import absolute_import

from checkout_sdk.api_client import ApiClient
from checkout_sdk.authorization_type import AuthorizationType
from checkout_sdk.checkout_configuration import CheckoutConfiguration
from checkout_sdk.client import Client
from checkout_sdk.sessions.session_secret_credentials import SessionSecretSdkCredentials
from checkout_sdk.sessions.sessions import SessionRequest, ChannelData, ThreeDsMethodCompletionRequest


class SessionsClient(Client):
    __SESSIONS_PATH = 'sessions'
    __COLLECT_DATA_PATH = 'collect-data'
    __COMPLETE_PATH = 'complete'
    __ISSUER_FINGERPRINT_PATH = 'issuer-fingerprint'

    def __init__(self, api_client: ApiClient, configuration: CheckoutConfiguration):
        super().__init__(api_client=api_client,
                         configuration=configuration,
                         authorization_type=AuthorizationType.OAUTH)

    def request_session(self, session_request: SessionRequest):
        return self._api_client.post(self.__SESSIONS_PATH, self._sdk_authorization(), session_request)

    def get_session_details(self, session_id: str, session_secret: str = None):
        return self._api_client.get(self.build_path(self.__SESSIONS_PATH, session_id),
                                    self.__custom_sdk_authorization(session_secret))

    def update_session(self, session_id: str, channel_data: ChannelData, session_secret: str = None):
        return self._api_client.put(self.build_path(self.__SESSIONS_PATH, session_id, self.__COLLECT_DATA_PATH),
                                    self.__custom_sdk_authorization(session_secret), channel_data)

    def complete_session(self, session_id: str, session_secret: str = None):
        return self._api_client.post(self.build_path(self.__SESSIONS_PATH, session_id, self.__COMPLETE_PATH),
                                     self.__custom_sdk_authorization(session_secret))

    def update_3ds_method_completion(self, session_id: str,
                                     three_ds_method_completion_request: ThreeDsMethodCompletionRequest,
                                     session_secret: str = None):
        return self._api_client.put(self.build_path(self.__SESSIONS_PATH, session_id, self.__ISSUER_FINGERPRINT_PATH),
                                    self.__custom_sdk_authorization(session_secret),
                                    three_ds_method_completion_request)

    def __custom_sdk_authorization(self, session_secret: str = None):
        if session_secret is None:
            return self._sdk_authorization()
        else:
            return SessionSecretSdkCredentials(session_secret).get_authorization(AuthorizationType.CUSTOM)



import inspect
import json


class JsonSerializer(json.JSONEncoder):
    _KEYS_TRANSFORMATIONS: dict = {'three_ds': '3ds',
                                   'account_holder_type': 'account-holder-type',
                                   'payment_network': 'payment-network',
                                   'from_': 'from',
                                   'if_match': 'if-match'}

    def default(self, obj):
        if hasattr(obj, 'to_json'):
            return self.default(obj.to_json())
        elif hasattr(obj, '__dict__'):
            props = dict(
                (key, value)
                for key, value in inspect.getmembers(obj)
                if not key.startswith('__')
                and not inspect.isabstract(value)
                and not inspect.isbuiltin(value)
                and not inspect.isfunction(value)
                and not inspect.isgenerator(value)
                and not inspect.isgeneratorfunction(value)
                and not inspect.ismethod(value)
                and not inspect.ismethoddescriptor(value)
                and not inspect.isroutine(value)
            )
            return self.default(self.apply_key_transformations(props))
        elif hasattr(obj, 'strftime'):
            return self.default(obj.replace(microsecond=0).isoformat())
        return obj

    def apply_key_transformations(self, props):
        for key in self._KEYS_TRANSFORMATIONS:
            if key in props:
                props[self._KEYS_TRANSFORMATIONS[key]] = props.pop(key)

        return props



from __future__ import absolute_import

from abc import abstractmethod

from requests import Session


class HttpClientBuilderInterface:

    @abstractmethod
    def get_client(self) -> Session:
        raise NotImplementedError()



import setuptools

with open('README.md', 'r') as f:
    long_description = f.read()

properties = {}
with open('checkout_sdk/properties.py') as p:
    exec(p.read(), properties)

setuptools.setup(
    name='checkout_sdk',
    version=properties['VERSION'],
    author='Checkout.com',
    author_email='support@checkout.com',
    description='Checkout.com Python SDK',
    long_description=long_description,
    long_description_content_type='text/markdown',
    zip_safe=False,
    url='https://github.com/checkout/checkout-sdk-python',
    license='MIT',
    packages=setuptools.find_packages(exclude=['tests', 'tests.*']),
    install_requires=[
        'requests >= 2.27.1'
    ],
    test_suite='tests',
    classifiers=[
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent'
    ]
)



from __future__ import absolute_import

from abc import abstractmethod

from checkout_sdk.authorization_type import AuthorizationType


class SdkCredentials:

    @abstractmethod
    def get_authorization(self, authorization_type: AuthorizationType):
        pass



from __future__ import absolute_import

from abc import ABCMeta

from checkout_sdk.api_client import ApiClient
from checkout_sdk.previous.checkout_api import CheckoutApi
from checkout_sdk.checkout_configuration import CheckoutConfiguration
from checkout_sdk.previous.previous_keys_credentials import PreviousKeysSdkCredentials
from checkout_sdk.static_keys_builder import StaticKeysBuilder, validate_secret_key, validate_public_key


class PreviousStaticKeys(StaticKeysBuilder, metaclass=ABCMeta):
    _PUBLIC_KEY_PATTERN = '^pk_(test_)?(\\w{8})-(\\w{4})-(\\w{4})-(\\w{4})-(\\w{12})$'
    _SECRET_KEY_PATTERN = '^sk_(test_)?(\\w{8})-(\\w{4})-(\\w{4})-(\\w{4})-(\\w{12})$'

    def secret_key(self, secret_key):
        super().secret_key(secret_key)
        return self

    def public_key(self, public_key):
        super().public_key(public_key)
        return self


class PreviousSdk(PreviousStaticKeys):

    def __init__(self):
        super().__init__()

    def build(self):
        validate_secret_key(self._SECRET_KEY_PATTERN, self._secret_key)
        validate_public_key(self._PUBLIC_KEY_PATTERN, self._public_key)
        if self._environment_subdomain is not None:
            configuration = CheckoutConfiguration(
                credentials=PreviousKeysSdkCredentials(secret_key=self._secret_key, public_key=self._public_key),
                environment=self._environment,
                http_client=self._http_client,
                environment_subdomain=self._environment_subdomain)
        else:
            configuration = CheckoutConfiguration(
                credentials=PreviousKeysSdkCredentials(secret_key=self._secret_key, public_key=self._public_key),
                environment=self._environment,
                http_client=self._http_client)
        return CheckoutApi(ApiClient(configuration, configuration.environment.base_uri), configuration)



from enum import Enum


class WebhookContentType(str, Enum):
    JSON = 'json'


class WebhookRequest:
    url: str
    active: bool
    headers: dict
    content_type: WebhookContentType
    event_types: list



from __future__ import absolute_import

import json
from abc import ABCMeta
from datetime import datetime, timedelta

import urllib3
from requests import HTTPError, Session, ConnectionError

from checkout_sdk.authorization_type import AuthorizationType
from checkout_sdk.environment import Environment
from checkout_sdk.exception import CheckoutAuthorizationException, CheckoutArgumentException
from checkout_sdk.oauth_access_token import OAuthAccessToken
from checkout_sdk.platform_type import PlatformType
from checkout_sdk.sdk_authorization import SdkAuthorization
from checkout_sdk.sdk_credentials import SdkCredentials


class OAuthSdkCredentials(SdkCredentials, metaclass=ABCMeta):
    __access_token: OAuthAccessToken = None
    __http_client: Session
    __scopes: list

    def __init__(self,
                 http_client: Session,
                 environment: Environment,
                 client_id: str,
                 client_secret: str,
                 authorization_uri: str,
                 scopes: list):
        self.validate_arguments(environment, client_id, client_secret, authorization_uri)
        self.__http_client = http_client
        self.__client_id = client_id
        self.__client_secret = client_secret
        self.__authorization_uri = authorization_uri if authorization_uri else environment.authorization_uri
        self.__scopes = scopes

    @staticmethod
    def validate_arguments(environment: Environment, client_id: str, client_secret: str, authorization_uri: str):
        if not client_id or not client_secret:
            raise CheckoutArgumentException(
                "Invalid OAuth 'client_id' or 'client_secret'")
        if not environment and not authorization_uri:
            raise CheckoutArgumentException(
                "Invalid configuration. Please specify an 'environment' or a specific OAuth 'authorization_uri'")

    @staticmethod
    def init(http_client: Session, environment: Environment, client_id, client_secret, authorization_uri, scopes: list):
        credentials = OAuthSdkCredentials(http_client, environment, client_id, client_secret, authorization_uri,
                                          scopes)
        credentials.get_access_token()
        return credentials

    def get_authorization(self, authorization_type: AuthorizationType):
        if authorization_type in (
                AuthorizationType.SECRET_KEY_OR_OAUTH, AuthorizationType.PUBLIC_KEY_OR_OAUTH, AuthorizationType.OAUTH):
            return SdkAuthorization(PlatformType.OAUTH, self.get_access_token().token)

        raise CheckoutAuthorizationException.invalid_authorization(authorization_type=authorization_type)

    def get_access_token(self):
        if self.__access_token is not None and self.__access_token.is_valid():
            return self.__access_token

        data = {
            'grant_type': 'client_credentials',
            'client_id': self.__client_id,
            'client_secret': self.__client_secret,
            'scope': str.join(' ', self.__scopes)
        }

        urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

        try:
            response = self.__http_client.request(method='POST', url=self.__authorization_uri,
                                                  data=data, verify=False, allow_redirects=False,
                                                  auth=(self.__client_id, self.__client_secret))
            response.raise_for_status()
        except HTTPError as err:
            errors = json.loads(err.response.text)
            message = 'OAuth client_credentials authentication failed with error: ({})'.format(errors['error'])
            raise CheckoutAuthorizationException(message) from err
        except ConnectionError as err:
            raise CheckoutAuthorizationException(
                'Unable to establish connection to host: ({})'.format(self.__authorization_uri)) from err

        response_json = response.json()
        self.__access_token = OAuthAccessToken(token=response_json['access_token'],
                                               token_type=response_json['token_type'],
                                               expiration_date=datetime.now() + timedelta(
                                                   seconds=response_json['expires_in']))
        return self.__access_token



from __future__ import absolute_import

from datetime import datetime
from enum import Enum

from checkout_sdk.common.common import AccountHolder, BankDetails, MarketplaceData, Address, Phone, CustomerRequest, \
    AccountHolderIdentification
from checkout_sdk.common.enums import PaymentSourceType, Currency, Country, AccountType, ChallengeIndicator
from checkout_sdk.sessions.sessions import DeliveryTimeframe


class AuthorizationType(str, Enum):
    FINAL = 'Final'
    ESTIMATED = 'Estimated'
    INCREMENTAL = 'Incremental'


class CaptureType(str, Enum):
    NON_FINAL = 'NonFinal'
    FINAL = 'Final'


class InstructionScheme(str, Enum):
    SWIFT = 'swift'
    LOCAL = 'local'
    INSTANT = 'instant'


class PaymentSenderType(str, Enum):
    INDIVIDUAL = 'individual'
    CORPORATE = 'corporate'
    INSTRUMENT = 'instrument'
    GOVERNMENT = 'government'


class PayoutSourceType(str, Enum):
    CURRENCY_ACCOUNT = 'currency_account'
    ENTITY = 'entity'


class PaymentType(str, Enum):
    REGULAR = 'Regular'
    RECURRING = 'Recurring'
    MOTO = 'MOTO'
    INSTALLMENT = 'Installment'
    UNSCHEDULED = 'Unscheduled'


class PaymentDestinationType(str, Enum):
    BANK_ACCOUNT = 'bank_account'
    CARD = 'card'
    ID = 'id'
    TOKEN = 'token'


class Exemption(str, Enum):
    LOW_VALUE = 'low_value'
    SECURE_CORPORATE_PAYMENT = 'secure_corporate_payment'
    TRUSTED_LISTING = 'trusted_listing'
    TRUSTED_LISTING_PROMPT = 'trusted_listing_prompt'
    TRANSACTION_RISK_ASSESSMENT = 'transaction_risk_assessment'
    THREE_DS_OUTAGE = '3ds_outage'
    SCA_DELEGATION = 'sca_delegation'
    OUT_OF_SCA_SCOPE = 'out_of_sca_scope'
    OTHER = 'other'
    LOW_RISK_PROGRAM = 'low_risk_program'
    DATA_SHARE = 'data_share'
    RECURRING_OPERATION = 'recurring_operation'


class ThreeDSFlowType(str, Enum):
    CHALLENGED = 'challenged'
    FRICTIONLESS = 'frictionless'
    FRICTIONLESS_DELEGATED = 'frictionless_delegated'


class MerchantInitiatedReason(str, Enum):
    DELAYED_CHARGE = 'Delayed_charge'
    RESUBMISSION = 'Resubmission'
    NO_SHOW = 'No_show'
    REAUTHORIZATION = 'Reauthorization'


class PreferredSchema(str, Enum):
    VISA = 'visa'
    MASTERCARD = 'mastercard'
    CARTES_BANCAIRES = 'cartes_bancaires'


class ProductType(str, Enum):
    QR_CODE = 'QR Code'
    IN_APP = 'In-App'
    OFFICIAL_ACCOUNT = 'Official Account'
    MINI_PROGRAM = 'Mini Program'


class TerminalType(str, Enum):
    APP = 'APP'
    WAP = 'WAP'
    WEB = 'WEB'


class OsType(str, Enum):
    ANDROID = 'ANDROID'
    IOS = 'IOS'


class ShippingPreference(str, Enum):
    NO_SHIPPING = 'NO_SHIPPING'
    SET_PROVIDED_ADDRESS = 'SET_PROVIDED_ADDRESS'
    GET_FROM_FILE = 'GET_FROM_FILE'


class UserAction(str, Enum):
    PAY_NOW = 'PAY_NOW'
    CONTINUE = 'CONTINUE'


class BillingPlanType(str, Enum):
    MERCHANT_INITIATED_BILLING = 'MERCHANT_INITIATED_BILLING'
    MERCHANT_INITIATED_BILLING_SINGLE_AGREEMENT = 'MERCHANT_INITIATED_BILLING_SINGLE_AGREEMENT'
    CHANNEL_INITIATED_BILLING = 'CHANNEL_INITIATED_BILLING'
    CHANNEL_INITIATED_BILLING_SINGLE_AGREEMENT = 'CHANNEL_INITIATED_BILLING_SINGLE_AGREEMENT'
    RECURRING_PAYMENTS = 'RECURRING_PAYMENTS'
    PRE_APPROVED_PAYMENTS = 'PRE_APPROVED_PAYMENTS'


class PanPreference(str, Enum):
    FPAN = 'fpan'
    DPAN = 'dpan'


class BillingDescriptor:
    name: str
    city: str
    reference: str


class PaymentInstruction:
    purpose: str
    charge_bearer: str
    repair: bool
    scheme: InstructionScheme
    quote_id: str


class PayoutBillingDescriptor:
    reference: str


# Payment Sender
class PaymentSender:
    type: PaymentSenderType

    def __init__(self, type_p: PaymentSenderType):
        self.type = type_p


class PaymentCorporateSender(PaymentSender):
    company_name: str
    address: Address
    reference: str
    reference_type: str
    source_of_funds: str
    identification: AccountHolderIdentification

    def __init__(self):
        super().__init__(PaymentSenderType.CORPORATE)


class PaymentGovermentSender(PaymentSender):
    company_name: str
    address: Address
    reference: str
    reference_type: str
    source_of_funds: str
    identification: AccountHolderIdentification

    def __init__(self):
        super().__init__(PaymentSenderType.GOVERNMENT)


class PaymentIndividualSender(PaymentSender):
    first_name: str
    middle_name: str
    last_name: str
    dob: str
    address: Address
    identification: AccountHolderIdentification
    reference: str
    reference_type: str
    source_of_funds: str
    date_of_birth: str
    country_of_birth: Country
    nationality: Country

    def __init__(self):
        super().__init__(PaymentSenderType.INDIVIDUAL)


class PaymentInstrumentSender(PaymentSender):

    def __init__(self):
        super().__init__(PaymentSenderType.INSTRUMENT)


# Payment Request Source
class PaymentRequestSource:
    type: PaymentSourceType

    def __init__(self, type_p: PaymentSourceType):
        self.type = type_p


class PaymentRequestCardSource(PaymentRequestSource):
    number: str
    expiry_month: int
    expiry_year: int
    name: str
    cvv: str
    stored: bool
    store_for_future_use: bool
    billing_address: Address
    phone: Phone
    account_holder: AccountHolder

    def __init__(self):
        super().__init__(PaymentSourceType.CARD)


class PaymentRequestTokenSource(PaymentRequestSource):
    token: str
    billing_address: Address
    phone: Phone
    stored: bool
    store_for_future_use: bool
    account_holder: AccountHolder

    def __init__(self):
        super().__init__(PaymentSourceType.TOKEN)


class PaymentRequestNetworkTokenSource(PaymentRequestSource):
    token: str
    expiry_month: int
    expiry_year: int
    token_type: str
    cryptogram: str
    eci: str
    stored: bool
    store_for_future_use: bool
    name: str
    cvv: str
    billing_address: Address
    phone: Phone
    account_holder: AccountHolder

    def __init__(self):
        super().__init__(PaymentSourceType.NETWORK_TOKEN)


class PaymentRequestIdSource(PaymentRequestSource):
    id: str
    cvv: str
    payment_method: str
    stored: bool
    store_for_future_use: bool
    account_holder: AccountHolder

    def __init__(self):
        super().__init__(PaymentSourceType.ID)


class RequestProviderTokenSource(PaymentRequestSource):
    payment_method: str
    token: str
    account_holder: AccountHolder

    def __init__(self):
        super().__init__(PaymentSourceType.PROVIDER_TOKEN)


class RequestBankAccountSource(PaymentRequestSource):
    payment_method: str
    account_type: str
    country: Country
    account_number: str
    bank_code: str
    account_holder: AccountHolder

    def __init__(self):
        super().__init__(PaymentSourceType.BANK_ACCOUNT)


class RequestCustomerSource(PaymentRequestSource):
    id: str
    account_holder: AccountHolder

    def __init__(self):
        super().__init__(PaymentSourceType.CUSTOMER)


class PaymentContextsShippingMethod(str, Enum):
    DIGITAL = 'Digital'
    PICK_UP = 'PickUp'
    BILLING_ADDRESS = 'BillingAddress'
    OTHER_ADDRESS = 'OtherAddress'


class ShippingDetails:
    first_name: str
    last_name: str
    email: str
    address: Address
    phone: Phone
    from_address_zip: str
    timeframe: DeliveryTimeframe
    method: PaymentContextsShippingMethod
    delay: int


class ThreeDsRequest:
    enabled: bool
    attempt_n3d: bool
    eci: str
    cryptogram: str
    xid: str
    version: str
    exemption: Exemption
    challenge_indicator: ChallengeIndicator
    allow_upgrade: bool
    status: str
    authentication_date: datetime
    authentication_amount: int
    flow_type: ThreeDSFlowType
    status_reason_code: str
    challenge_cancel_reason: str
    score: str
    cryptogram_algorithm: str
    authentication_id: str


class RiskRequest:
    enabled: bool
    device_session_id: str


class PaymentRecipient:
    dob: str
    account_number: str
    address: Address
    zip: str
    first_name: str
    last_name: str
    country: Country


class Payer:
    name: str
    email: str
    document: str


class Installments:
    count: str


class DLocalProcessingSettings:
    country: Country
    payer: Payer
    installments: Installments


class SenderInformation:
    reference: str
    firstName: str
    lastName: str
    dob: str
    address: str
    city: str
    state: str
    country: str
    postalCode: str
    sourceOfFunds: str


class PartnerCustomerRiskData:
    key: str
    value: str


class AccommodationAddress:
    address_line1: str
    zip: str


class AccommodationGuest:
    first_name: str
    last_name: str
    date_of_birth: str


class AccommodationRoom:
    rate: str
    number_of_nights_at_room_rate: str


class AccommodationData:
    name: str
    booking_reference: str
    check_in_date: str
    check_out_date: str
    address: AccommodationAddress
    state: str
    country: str
    city: str
    number_of_rooms: int
    guests: list  # AccommodationGuest
    room: list  # AccommodationRoom


class ProcessingSettings:
    order_id: str
    tax_amount: int
    discount_amount: int
    duty_amount: int
    shipping_amount: int
    shipping_tax_amount: int
    aft: bool
    preferred_scheme: PreferredSchema
    merchant_initiated_reason: MerchantInitiatedReason
    campaign_id: int
    product_type: ProductType
    open_id: str
    original_order_amount: int
    receipt_id: str
    terminal_type: TerminalType
    os_type: OsType
    invoice_id: str
    brand_name: str
    locale: str
    shipping_preference: ShippingPreference
    user_action: UserAction
    set_transaction_context: list  # dict
    airline_data: list  # AirlineData
    otp_value: str
    purchase_country: Country
    custom_payment_method_ids: list  # string
    merchant_callback_url: str
    line_of_business: str
    shipping_delay: int
    shipping_info: list  # ShippingInfo
    dlocal: DLocalProcessingSettings
    senderInformation: SenderInformation
    purpose: str
    partner_customer_risk_data: list  # PartnerCustomerRiskData
    accommodation_data: list  # AccommodationData
    surcharge_amount: int
    pan_preference: PanPreference
    provision_network_token: bool


class Product:
    name: str
    quantity: int
    unit_price: int
    reference: str
    commodity_code: str
    unit_of_measure: str
    total_amount: int
    tax_amount: int
    discount_amount: int
    wxpay_goods_id: str
    image_url: str
    url: str
    sku: str


class PaymentCustomerRequest(CustomerRequest):
    tax_number: str


class PaymentSegment:
    brand: str
    business_category: str
    market: str


class PaymentRetryRequest:
    enabled: bool
    max_attempts: int
    end_after_days: int


# Request Payment
class PartialAuthorization:
    enabled: bool


class PaymentRequest:
    payment_context_id: str
    source: PaymentRequestSource
    amount: int
    currency: Currency
    payment_type: PaymentType
    merchant_initiated: bool
    reference: str
    description: str
    authorization_type: AuthorizationType
    partial_authorization: PartialAuthorization
    capture: bool
    capture_on: datetime
    customer: PaymentCustomerRequest
    billing_descriptor: BillingDescriptor
    shipping: ShippingDetails
    segment: PaymentSegment
    three_ds: ThreeDsRequest
    processing_channel_id: str
    previous_payment_id: str
    risk: RiskRequest
    success_url: str
    failure_url: str
    payment_ip: str
    sender: PaymentSender
    recipient: PaymentRecipient
    # @deprecated marketplace property will be removed in the future, and should be used amount_allocations instead
    marketplace: MarketplaceData
    amount_allocations: list  # values of AmountAllocations
    processing: ProcessingSettings
    metadata: dict
    items: list  # payments.Product
    retry: PaymentRetryRequest
    instruction: PaymentInstruction


# Payout Request Source
class PayoutRequestSource:
    type: PayoutSourceType

    def __init__(self, type_p: PayoutSourceType):
        self.type = type_p


class PayoutRequestCurrencyAccountSource(PayoutRequestSource):
    id: str

    def __init__(self):
        super().__init__(PayoutSourceType.CURRENCY_ACCOUNT)


class PayoutRequestEntitySource(PayoutRequestSource):
    id: str

    def __init__(self):
        super().__init__(PayoutSourceType.ENTITY)


# Payment Request Destination
class PaymentRequestDestination:
    type: PaymentDestinationType

    def __init__(self, type_p: PaymentDestinationType):
        self.type = type_p


class PaymentBankAccountDestination(PaymentRequestDestination):
    account_type: AccountType
    account_number: str
    bank_code: str
    branch_code: str
    iban: str
    swift_bic: str
    country: Country
    account_holder: AccountHolder
    bank: BankDetails

    def __init__(self):
        super().__init__(PaymentDestinationType.BANK_ACCOUNT)


class PaymentRequestIdDestination(PaymentRequestDestination):
    id: str

    account_holder: AccountHolder

    def __init__(self):
        super().__init__(PaymentDestinationType.ID)


# Request Payout
class PayoutRequest:
    source: PayoutRequestSource
    destination: PaymentRequestDestination
    amount: int
    currency: Currency
    reference: str
    billing_descriptor: PayoutBillingDescriptor
    sender: PaymentSender
    instruction: PaymentInstruction
    processing_channel_id: str


# Query
class PaymentsQueryFilter:
    limit: int
    skip: int
    reference: str


# Captures
class CaptureRequest:
    amount: int
    capture_type: CaptureType
    reference: str
    customer: PaymentCustomerRequest
    description: str
    billing_descriptor: BillingDescriptor
    shipping: ShippingDetails
    items: list  # payments.Product
    # @deprecated marketplace property will be removed in the future, and should be used amount_allocations instead
    marketplace: MarketplaceData
    amount_allocations: list  # values of AmountAllocations
    processing: ProcessingSettings
    metadata: dict


# Authorization
class AuthorizationRequest:
    amount: int
    reference: str
    metadata: dict


# Refunds
class RefundRequest:
    amount: int
    reference: str
    metadata: dict
    # Not available on Previous
    amount_allocations: list  # values of AmountAllocations


# Voids
class VoidRequest:
    reference: str
    metadata: dict


class BillingPlan:
    type: BillingPlanType
    skip_shipping_address: bool
    immutable_shipping_address: bool


class FawryProduct:
    product_id: str
    quantity: int
    price: int
    description: str


class PaymentMethodDetails:
    display_name: str
    type: str
    network: str



from __future__ import absolute_import

from enum import Enum

from checkout_sdk.common.common import Address, Phone


class TokenType(str, Enum):
    CARD = 'card'
    APPLE_PAY = 'applepay'
    GOOGLE_PAY = 'googlepay'


class CardTokenRequest:
    type: TokenType
    number: str
    expiry_month: int
    expiry_year: int
    name: str
    cvv: str
    billing_address: Address
    phone: Phone

    def __init__(self):
        self.type = TokenType.CARD


class WalletTokenRequest:
    type: str

    def __init__(self, token_type: TokenType):
        self.type = token_type


class GooglePayTokenData:
    signature: str
    protocolVersion: str
    signedMessage: str


class GooglePayTokenRequest(WalletTokenRequest):
    token_data: GooglePayTokenData

    def __init__(self):
        super().__init__(TokenType.GOOGLE_PAY)


class ApplePayTokenData:
    version: str
    data: str
    signature: str
    header: dict


class ApplePayTokenRequest(WalletTokenRequest):
    token_data: ApplePayTokenData

    def __init__(self):
        super().__init__(TokenType.APPLE_PAY)



from __future__ import absolute_import

from checkout_sdk.api_client import ApiClient
from checkout_sdk.authorization_type import AuthorizationType
from checkout_sdk.checkout_configuration import CheckoutConfiguration
from checkout_sdk.client import Client


class SepaClient(Client):
    __APMS_PATH = 'apms'
    __SEPA_MANDATES_PATH = 'sepa/mandates'
    __PPRO_PATH = 'ppro'
    __CANCEL_PATH = 'cancel'

    def __init__(self, api_client: ApiClient, configuration: CheckoutConfiguration):
        super().__init__(api_client=api_client,
                         configuration=configuration,
                         authorization_type=AuthorizationType.SECRET_KEY)

    def get_mandate(self, mandate_id: str):
        return self._api_client.get(self.build_path(self.__SEPA_MANDATES_PATH, mandate_id), self._sdk_authorization())

    def cancel_mandate(self, mandate_id: str):
        return self._api_client.post(self.build_path(self.__SEPA_MANDATES_PATH, mandate_id, self.__CANCEL_PATH),
                                     self._sdk_authorization())

    def get_mandate_via_ppro(self, mandate_id: str):
        return self._api_client.get(self.build_path(self.__APMS_PATH, self.__PPRO_PATH,
                                                    self.__SEPA_MANDATES_PATH, mandate_id),
                                    self._sdk_authorization())

    def cancel_mandate_via_ppro(self, mandate_id: str):
        return self._api_client.post(self.build_path(self.__APMS_PATH, self.__PPRO_PATH,
                                                     self.__SEPA_MANDATES_PATH, mandate_id, self.__CANCEL_PATH),
                                     self._sdk_authorization())



from __future__ import absolute_import

from checkout_sdk.api_client import ApiClient
from checkout_sdk.authorization_type import AuthorizationType
from checkout_sdk.checkout_configuration import CheckoutConfiguration
from checkout_sdk.client import Client
from checkout_sdk.workflows.workflows import CreateWorkflowRequest, UpdateWorkflowRequest, WorkflowActionRequest, \
    WorkflowConditionRequest, ReflowRequest, EventTypesRequest


class WorkflowsClient(Client):
    __WORKFLOWS_PATH = 'workflows'
    __ACTIONS_PATH = 'actions'
    __CONDITIONS_PATH = 'conditions'
    __EVENT_TYPES_PATH = 'event-types'
    __EVENTS_PATH = 'events'
    __SUBJECT_PATH = 'subject'
    __REFLOW_PATH = 'reflow'
    __WORKFLOW_PATH = 'workflow'
    __TEST_PATH = 'test'

    def __init__(self, api_client: ApiClient, configuration: CheckoutConfiguration):
        super().__init__(api_client=api_client,
                         configuration=configuration,
                         authorization_type=AuthorizationType.SECRET_KEY_OR_OAUTH)

    def get_workflows(self):
        return self._api_client.get(self.__WORKFLOWS_PATH,
                                    self._sdk_authorization())

    def create_workflow(self, create_workflow_request: CreateWorkflowRequest):
        return self._api_client.post(self.__WORKFLOWS_PATH,
                                     self._sdk_authorization(),
                                     create_workflow_request)

    def get_workflow(self, workflow_id: str):
        return self._api_client.get(self.build_path(self.__WORKFLOWS_PATH, workflow_id),
                                    self._sdk_authorization())

    def remove_workflow(self, workflow_id: str):
        return self._api_client.delete(self.build_path(self.__WORKFLOWS_PATH, workflow_id),
                                       self._sdk_authorization())

    def update_workflow(self, workflow_id: str, update_workflow_request: UpdateWorkflowRequest):
        return self._api_client.patch(self.build_path(self.__WORKFLOWS_PATH, workflow_id),
                                      self._sdk_authorization(),
                                      update_workflow_request)

    def add_workflow_action(self, workflow_id: str, workflow_action_request: WorkflowActionRequest):
        return self._api_client.post(self.build_path(self.__WORKFLOWS_PATH, workflow_id, self.__ACTIONS_PATH),
                                     self._sdk_authorization(),
                                     workflow_action_request)

    def update_workflow_action(self, workflow_id: str, action_id: str, workflow_action_request: WorkflowActionRequest):
        return self._api_client.put(self.build_path(self.__WORKFLOWS_PATH, workflow_id, self.__ACTIONS_PATH, action_id),
                                    self._sdk_authorization(),
                                    workflow_action_request)

    def remove_workflow_action(self, workflow_id: str, action_id: str):
        return self._api_client.delete(
            self.build_path(self.__WORKFLOWS_PATH, workflow_id, self.__ACTIONS_PATH, action_id),
            self._sdk_authorization())

    def add_workflow_condition(self, workflow_id: str, workflow_condition_request: WorkflowConditionRequest):
        return self._api_client.post(
            self.build_path(self.__WORKFLOWS_PATH, workflow_id, self.__CONDITIONS_PATH),
            self._sdk_authorization(),
            workflow_condition_request)

    def update_workflow_condition(self, workflow_id: str, condition_id: str,
                                  workflow_condition_request: WorkflowConditionRequest):
        return self._api_client.put(
            self.build_path(self.__WORKFLOWS_PATH, workflow_id, self.__CONDITIONS_PATH, condition_id),
            self._sdk_authorization(),
            workflow_condition_request)

    def remove_workflow_condition(self, workflow_id: str, condition_id: str):
        return self._api_client.delete(
            self.build_path(self.__WORKFLOWS_PATH, workflow_id, self.__CONDITIONS_PATH, condition_id),
            self._sdk_authorization())

    def test_workflow(self, workflow_id: str, event_types_request: EventTypesRequest):
        return self._api_client.post(self.build_path(self.__WORKFLOWS_PATH, workflow_id, self.__TEST_PATH),
                                     self._sdk_authorization(),
                                     event_types_request)

    def get_event_types(self):
        return self._api_client.get(self.build_path(self.__WORKFLOWS_PATH, self.__EVENT_TYPES_PATH),
                                    self._sdk_authorization())

    def get_event(self, event_id: str):
        return self._api_client.get(
            self.build_path(self.__WORKFLOWS_PATH, self.__EVENTS_PATH, event_id),
            self._sdk_authorization())

    def get_action_invocations(self, event_id: str, action_id: str):
        return self._api_client.get(
            self.build_path(self.__WORKFLOWS_PATH, self.__EVENTS_PATH, event_id, self.__ACTIONS_PATH, action_id),
            self._sdk_authorization())

    def reflow_by_event(self, event_id: str):
        return self._api_client.post(
            self.build_path(self.__WORKFLOWS_PATH, self.__EVENTS_PATH, event_id, self.__REFLOW_PATH),
            self._sdk_authorization())

    def reflow_by_event_and_workflow(self, event_id: str, workflow_id: str):
        return self._api_client.post(
            self.build_path(self.__WORKFLOWS_PATH, self.__EVENTS_PATH, event_id, self.__WORKFLOW_PATH, workflow_id,
                            self.__REFLOW_PATH),
            self._sdk_authorization())

    def reflow(self, reflow_request: ReflowRequest):
        return self._api_client.post(self.build_path(self.__WORKFLOWS_PATH, self.__EVENTS_PATH, self.__REFLOW_PATH),
                                     self._sdk_authorization(),
                                     reflow_request)

    def get_subject_events(self, subject_id: str):
        return self._api_client.get(
            self.build_path(self.__WORKFLOWS_PATH, self.__EVENTS_PATH, self.__SUBJECT_PATH, subject_id),
            self._sdk_authorization())

    def reflow_by_subject(self, subject_id: str):
        return self._api_client.post(
            self.build_path(self.__WORKFLOWS_PATH, self.__EVENTS_PATH, self.__SUBJECT_PATH, subject_id,
                            self.__REFLOW_PATH),
            self._sdk_authorization())

    def reflow_by_subject_and_workflow(self, subject_id: str, workflow_id: str):
        return self._api_client.post(
            self.build_path(self.__WORKFLOWS_PATH, self.__EVENTS_PATH, self.__SUBJECT_PATH, subject_id,
                            self.__WORKFLOW_PATH, workflow_id, self.__REFLOW_PATH),
            self._sdk_authorization())



from __future__ import absolute_import

from checkout_sdk.api_client import ApiClient
from checkout_sdk.authorization_type import AuthorizationType
from checkout_sdk.checkout_configuration import CheckoutConfiguration
from checkout_sdk.client import Client
from checkout_sdk.payments.hosted.hosted_payments import HostedPaymentsSessionRequest


class HostedPaymentsClient(Client):
    __HOSTED_PAYMENTS_PATH = 'hosted-payments'

    def __init__(self, api_client: ApiClient, configuration: CheckoutConfiguration):
        super().__init__(api_client=api_client,
                         configuration=configuration,
                         authorization_type=AuthorizationType.SECRET_KEY)

    def get_hosted_payments_page_details(self, hosted_payment_id: str):
        return self._api_client.get(self.build_path(self.__HOSTED_PAYMENTS_PATH, hosted_payment_id),
                                    self._sdk_authorization())

    def create_hosted_payments_page_session(self, hosted_payments_session_request: HostedPaymentsSessionRequest):
        return self._api_client.post(self.__HOSTED_PAYMENTS_PATH, self._sdk_authorization(),
                                     hosted_payments_session_request)



from __future__ import absolute_import

from checkout_sdk.api_client import ApiClient
from checkout_sdk.authorization_type import AuthorizationType
from checkout_sdk.checkout_configuration import CheckoutConfiguration
from checkout_sdk.client import Client
from checkout_sdk.metadata.metadata import CardMetadataRequest


class CardMetadataClient(Client):
    __CARD_METADATA_PATH = 'metadata/card'

    def __init__(self, api_client: ApiClient, configuration: CheckoutConfiguration):
        super().__init__(api_client=api_client,
                         configuration=configuration,
                         authorization_type=AuthorizationType.SECRET_KEY_OR_OAUTH)

    def request_card_metadata(self, card_metadata_request: CardMetadataRequest):
        return self._api_client.post(self.__CARD_METADATA_PATH, self._sdk_authorization(), card_metadata_request)
